@@ -3,6 +3,7 @@ package peco
 import (
 	"bytes"
 	"fmt"
+	"math"
 	"os"
 	"strconv"
 	"unicode"
@@ -10,7 +11,7 @@ import (
 	"context"
 
 	"github.com/google/btree"
-	"github.com/lestrrat/go-pdebug"
+	"github.com/lestrrat-go/pdebug"
 	"github.com/nsf/termbox-go"
 	"github.com/peco/peco/internal/keyseq"
 	"github.com/peco/peco/internal/util"
@@ -52,7 +53,7 @@ func (a ActionFunc) RegisterKeySequence(name string, k keyseq.KeyList) {
 
 func wrapDeprecated(fn func(context.Context, *Peco, termbox.Event), oldName, newName string) ActionFunc {
 	return ActionFunc(func(ctx context.Context, state *Peco, e termbox.Event) {
-		state.Hub().SendStatusMsg(fmt.Sprintf("%s is deprecated. Use %s", oldName, newName))
+		state.Hub().SendStatusMsg(ctx, fmt.Sprintf("%s is deprecated. Use %s", oldName, newName))
 		fn(ctx, state, e)
 	})
 }
@@ -105,6 +106,9 @@ func init() {
 	ActionFunc(doScrollLeft).Register("ScrollLeft")
 	ActionFunc(doScrollRight).Register("ScrollRight")
 
+	ActionFunc(doScrollFirstItem).Register("ScrollFirstItem", termbox.KeyHome)
+	ActionFunc(doScrollLastItem).Register("ScrollLastItem", termbox.KeyEnd)
+
 	ActionFunc(doToggleSelection).Register("ToggleSelection")
 	ActionFunc(doToggleSelectionAndSelectNext).Register(
 		"ToggleSelectionAndSelectNext",
@@ -124,6 +128,11 @@ func init() {
 	ActionFunc(doRefreshScreen).Register("RefreshScreen", termbox.KeyCtrlL)
 	ActionFunc(doToggleSingleKeyJump).Register("ToggleSingleKeyJump")
 
+	ActionFunc(doToggleViewArround).Register("ViewArround", termbox.KeyCtrlV)
+
+	ActionFunc(doGoToNextSelection).Register("GoToNextSelection", termbox.KeyCtrlK)
+	ActionFunc(doGoToPreviousSelection).Register("	doGoToPreviousSelection", termbox.KeyCtrlJ)
+
 	ActionFunc(doKonamiCommand).RegisterKeySequence(
 		"KonamiCommand",
 		keyseq.KeyList{
@@ -169,9 +178,9 @@ func doAcceptChar(ctx context.Context, state *Peco, e termbox.Event) {
 	c.Move(1)
 
 	h := state.Hub()
-	h.SendDrawPrompt() // Update prompt before running query
+	h.SendDrawPrompt(ctx) // Update prompt before running query
 
-	state.ExecQuery()
+	state.ExecQuery(nil)
 }
 
 func doRotateFilter(ctx context.Context, state *Peco, e termbox.Event) {
@@ -183,10 +192,10 @@ func doRotateFilter(ctx context.Context, state *Peco, e termbox.Event) {
 	filters := state.Filters()
 	filters.Rotate()
 
-	if state.ExecQuery() {
+	if state.ExecQuery(nil) {
 		return
 	}
-	state.Hub().SendDrawPrompt()
+	state.Hub().SendDrawPrompt(ctx)
 }
 
 func doBackToInitialFilter(ctx context.Context, state *Peco, e termbox.Event) {
@@ -198,10 +207,10 @@ func doBackToInitialFilter(ctx context.Context, state *Peco, e termbox.Event) {
 	filters := state.Filters()
 	filters.Reset()
 
-	if state.ExecQuery() {
+	if state.ExecQuery(nil) {
 		return
 	}
-	state.Hub().SendDrawPrompt()
+	state.Hub().SendDrawPrompt(ctx)
 }
 
 func doToggleSelection(ctx context.Context, state *Peco, _ termbox.Event) {
@@ -247,7 +256,7 @@ func doCancelRangeMode(ctx context.Context, state *Peco, _ termbox.Event) {
 
 func doSelectNone(ctx context.Context, state *Peco, _ termbox.Event) {
 	state.Selection().Reset()
-	state.Hub().SendDraw(&DrawOptions{DisableCache: true})
+	state.Hub().SendDraw(ctx, &DrawOptions{DisableCache: true})
 }
 
 func doSelectAll(ctx context.Context, state *Peco, _ termbox.Event) {
@@ -261,7 +270,7 @@ func doSelectAll(ctx context.Context, state *Peco, _ termbox.Event) {
 			selection.Remove(l)
 		}
 	}
-	state.Hub().SendDraw(nil)
+	state.Hub().SendDraw(ctx, nil)
 }
 
 func doSelectVisible(ctx context.Context, state *Peco, _ termbox.Event) {
@@ -283,7 +292,7 @@ func doSelectVisible(ctx context.Context, state *Peco, _ termbox.Event) {
 		l.SetDirty(true)
 		selection.Add(l)
 	}
-	state.Hub().SendDraw(nil)
+	state.Hub().SendDraw(ctx, nil)
 }
 
 type errCollectResults struct{}
@@ -323,12 +332,12 @@ func doFinish(ctx context.Context, state *Peco, _ termbox.Event) {
 	})
 
 	var err error
-	state.Hub().SendStatusMsg("Executing " + ccarg)
+	state.Hub().SendStatusMsg(ctx, "Executing " + ccarg)
 	cmd := util.Shell(ccarg)
 	cmd.Stdin = &stdin
 	cmd.Stdout = state.Stdout
 	cmd.Stderr = state.Stderr
-	// Setup some enviroment variables. Start with a copy of the current
+	// Setup some environment variables. Start with a copy of the current
 	// environment...
 	env := os.Environ()
 
@@ -356,7 +365,7 @@ func doFinish(ctx context.Context, state *Peco, _ termbox.Event) {
 
 	err = cmd.Run()
 	state.screen.Resume()
-	state.Hub().SendDraw(&DrawOptions{DisableCache: true})
+	state.Hub().SendDraw(ctx, &DrawOptions{DisableCache: true})
 	if err != nil {
 		// bail out, or otherwise the user cannot know what happened
 		state.Exit(errors.Wrap(err, `failed to execute command`))
@@ -389,7 +398,7 @@ func doSelectDown(ctx context.Context, state *Peco, e termbox.Event) {
 		g := pdebug.Marker("doSelectDown")
 		defer g.End()
 	}
-	state.Hub().SendPaging(ToLineBelow)
+	state.Hub().SendPaging(ctx, ToLineBelow)
 }
 
 func doSelectUp(ctx context.Context, state *Peco, e termbox.Event) {
@@ -397,28 +406,36 @@ func doSelectUp(ctx context.Context, state *Peco, e termbox.Event) {
 		g := pdebug.Marker("doSelectUp")
 		defer g.End()
 	}
-	state.Hub().SendPaging(ToLineAbove)
+	state.Hub().SendPaging(ctx, ToLineAbove)
 }
 
 func doScrollPageUp(ctx context.Context, state *Peco, e termbox.Event) {
-	state.Hub().SendPaging(ToScrollPageUp)
+	state.Hub().SendPaging(ctx, ToScrollPageUp)
 }
 
 func doScrollPageDown(ctx context.Context, state *Peco, e termbox.Event) {
-	state.Hub().SendPaging(ToScrollPageDown)
+	state.Hub().SendPaging(ctx, ToScrollPageDown)
 }
 
 func doScrollLeft(ctx context.Context, state *Peco, e termbox.Event) {
-	state.Hub().SendPaging(ToScrollLeft)
+	state.Hub().SendPaging(ctx, ToScrollLeft)
 }
 
 func doScrollRight(ctx context.Context, state *Peco, e termbox.Event) {
-	state.Hub().SendPaging(ToScrollRight)
+	state.Hub().SendPaging(ctx, ToScrollRight)
+}
+
+func doScrollFirstItem(ctx context.Context, state *Peco, e termbox.Event) {
+	state.Hub().SendPaging(ctx, ToScrollFirstItem)
+}
+
+func doScrollLastItem(ctx context.Context, state *Peco, e termbox.Event) {
+	state.Hub().SendPaging(ctx, ToScrollLastItem)
 }
 
 func doToggleSelectionAndSelectNext(ctx context.Context, state *Peco, e termbox.Event) {
 	toplevel, _ := ctx.Value(isTopLevelActionCall).(bool)
-	state.Hub().Batch(func() {
+	state.Hub().Batch(ctx, func(ctx context.Context) {
 		ctx = context.WithValue(ctx, isTopLevelActionCall, false)
 		doToggleSelection(ctx, state, e)
 		// XXX This is sucky. Fix later
@@ -452,7 +469,7 @@ func doInvertSelection(ctx context.Context, state *Peco, _ termbox.Event) {
 		}
 	}
 
-	state.Hub().SendDraw(nil)
+	state.Hub().SendDraw(ctx, nil)
 }
 
 func doDeleteBackwardWord(ctx context.Context, state *Peco, _ termbox.Event) {
@@ -492,17 +509,17 @@ func doDeleteBackwardWord(ctx context.Context, state *Peco, _ termbox.Event) {
 		q.DeleteRange(0, start)
 		c.SetPos(0)
 	}
-	if state.ExecQuery() {
+	if state.ExecQuery(nil) {
 		return
 	}
-	state.Hub().SendDrawPrompt()
+	state.Hub().SendDrawPrompt(ctx)
 }
 
 func doForwardWord(ctx context.Context, state *Peco, _ termbox.Event) {
 	if state.Caret().Pos() >= state.Query().Len() {
 		return
 	}
-	defer state.Hub().SendDrawPrompt()
+	defer state.Hub().SendDrawPrompt(ctx)
 
 	foundSpace := false
 	q := state.Query()
@@ -531,7 +548,7 @@ func doBackwardWord(ctx context.Context, state *Peco, _ termbox.Event) {
 	if c.Pos() == 0 {
 		return
 	}
-	defer state.Hub().SendDrawPrompt()
+	defer state.Hub().SendDrawPrompt(ctx)
 
 	if c.Pos() >= q.Len() {
 		c.Move(-1)
@@ -575,7 +592,7 @@ func doForwardChar(ctx context.Context, state *Peco, _ termbox.Event) {
 		return
 	}
 	c.Move(1)
-	state.Hub().SendDrawPrompt()
+	state.Hub().SendDrawPrompt(ctx)
 }
 
 func doBackwardChar(ctx context.Context, state *Peco, _ termbox.Event) {
@@ -584,7 +601,7 @@ func doBackwardChar(ctx context.Context, state *Peco, _ termbox.Event) {
 		return
 	}
 	c.Move(-1)
-	state.Hub().SendDrawPrompt()
+	state.Hub().SendDrawPrompt(ctx)
 }
 
 func doDeleteForwardWord(ctx context.Context, state *Peco, _ termbox.Event) {
@@ -617,20 +634,20 @@ func doDeleteForwardWord(ctx context.Context, state *Peco, _ termbox.Event) {
 		}
 	}
 
-	if state.ExecQuery() {
+	if state.ExecQuery(nil) {
 		return
 	}
-	state.Hub().SendDrawPrompt()
+	state.Hub().SendDrawPrompt(ctx)
 }
 
 func doBeginningOfLine(ctx context.Context, state *Peco, _ termbox.Event) {
 	state.Caret().SetPos(0)
-	state.Hub().SendDrawPrompt()
+	state.Hub().SendDrawPrompt(ctx)
 }
 
 func doEndOfLine(ctx context.Context, state *Peco, _ termbox.Event) {
 	state.Caret().SetPos(state.Query().Len())
-	state.Hub().SendDrawPrompt()
+	state.Hub().SendDrawPrompt(ctx)
 }
 
 func doEndOfFile(ctx context.Context, state *Peco, e termbox.Event) {
@@ -645,10 +662,10 @@ func doKillBeginningOfLine(ctx context.Context, state *Peco, _ termbox.Event) {
 	q := state.Query()
 	q.DeleteRange(0, state.Caret().Pos())
 	state.Caret().SetPos(0)
-	if state.ExecQuery() {
+	if state.ExecQuery(nil) {
 		return
 	}
-	state.Hub().SendDrawPrompt()
+	state.Hub().SendDrawPrompt(ctx)
 }
 
 func doKillEndOfLine(ctx context.Context, state *Peco, _ termbox.Event) {
@@ -658,15 +675,15 @@ func doKillEndOfLine(ctx context.Context, state *Peco, _ termbox.Event) {
 
 	q := state.Query()
 	q.DeleteRange(state.Caret().Pos(), q.Len())
-	if state.ExecQuery() {
+	if state.ExecQuery(nil) {
 		return
 	}
-	state.Hub().SendDrawPrompt()
+	state.Hub().SendDrawPrompt(ctx)
 }
 
 func doDeleteAll(ctx context.Context, state *Peco, _ termbox.Event) {
 	state.Query().Reset()
-	state.ExecQuery()
+	state.ExecQuery(nil)
 }
 
 func doDeleteForwardChar(ctx context.Context, state *Peco, _ termbox.Event) {
@@ -679,10 +696,10 @@ func doDeleteForwardChar(ctx context.Context, state *Peco, _ termbox.Event) {
 	pos := c.Pos()
 	q.DeleteRange(pos, pos+1)
 
-	if state.ExecQuery() {
+	if state.ExecQuery(nil) {
 		return
 	}
-	state.Hub().SendDrawPrompt()
+	state.Hub().SendDrawPrompt(ctx)
 }
 
 func doDeleteBackwardChar(ctx context.Context, state *Peco, e termbox.Event) {
@@ -718,15 +735,15 @@ func doDeleteBackwardChar(ctx context.Context, state *Peco, e termbox.Event) {
 	}
 	c.SetPos(pos - 1)
 
-	if state.ExecQuery() {
+	if state.ExecQuery(nil) {
 		return
 	}
 
-	state.Hub().SendDrawPrompt()
+	state.Hub().SendDrawPrompt(ctx)
 }
 
 func doRefreshScreen(ctx context.Context, state *Peco, _ termbox.Event) {
-	state.Hub().SendDraw(&DrawOptions{DisableCache: true})
+	state.Hub().SendDraw(ctx, &DrawOptions{DisableCache: true})
 }
 
 func doToggleQuery(ctx context.Context, state *Peco, _ termbox.Event) {
@@ -742,14 +759,14 @@ func doToggleQuery(ctx context.Context, state *Peco, _ termbox.Event) {
 		q.SaveQuery()
 	}
 
-	if state.ExecQuery() {
+	if state.ExecQuery(nil) {
 		return
 	}
-	state.Hub().SendDrawPrompt()
+	state.Hub().SendDrawPrompt(ctx)
 }
 
 func doKonamiCommand(ctx context.Context, state *Peco, e termbox.Event) {
-	state.Hub().SendStatusMsg("All your filters are belongs to us")
+	state.Hub().SendStatusMsg(ctx, "All your filters are belongs to us")
 }
 
 func doToggleSingleKeyJump(ctx context.Context, state *Peco, e termbox.Event) {
@@ -760,6 +777,125 @@ func doToggleSingleKeyJump(ctx context.Context, state *Peco, e termbox.Event) {
 	state.ToggleSingleKeyJumpMode()
 }
 
+func doToggleViewArround(ctx context.Context, state *Peco, e termbox.Event) {
+	if pdebug.Enabled {
+		g := pdebug.Marker("doToggleViewArround")
+		defer g.End()
+	}
+	q := state.Query()
+
+	if q.Len() > 0 {
+		l, err := state.CurrentLineBuffer().LineAt(state.Location().LineNumber())
+		if err != nil {
+			return
+		}
+		currentLine := l.ID()
+
+		doDeleteAll(ctx, state, e)
+		state.Hub().SendPaging(ctx, JumpToLineRequest(currentLine))
+	}
+}
+
+func doGoToNextSelection(ctx context.Context, state *Peco, _ termbox.Event) {
+	if pdebug.Enabled {
+		g := pdebug.Marker("doGoToNextSelection")
+		defer g.End()
+	}
+
+	selection := state.Selection()
+
+	if selection.Len() == 0 {
+		state.Hub().SendStatusMsg(ctx, "No Selection")
+		return
+	}
+
+	b := state.CurrentLineBuffer()
+	l, err := b.LineAt(state.Location().LineNumber())
+	if err != nil {
+		return
+	}
+	currentLine := l.ID()
+	nextLine := uint64(math.MaxUint64)
+	firstLine := uint64(math.MaxUint64)
+	found := false
+
+	selection.Ascend(func(it btree.Item) bool {
+		selectedLine := it.(line.Line)
+		if selectedLine.ID() > currentLine {
+			if selectedLine.ID() < nextLine {
+				nextLine = selectedLine.ID()
+				found = true
+			}
+		}
+
+		if selectedLine.ID() <= firstLine {
+			firstLine = selectedLine.ID()
+		}
+
+		return true
+	})
+
+	if found {
+		state.Hub().SendStatusMsg(ctx, "Next Selection")
+		state.Hub().SendPaging(ctx, ToScrollFirstItem)
+		state.Hub().SendPaging(ctx, JumpToLineRequest(nextLine))
+	} else {
+		state.Hub().SendStatusMsg(ctx, "Next Selection (first)")
+		state.Hub().SendPaging(ctx, ToScrollFirstItem)
+		state.Hub().SendPaging(ctx, JumpToLineRequest(firstLine))
+	}
+}
+
+func doGoToPreviousSelection(ctx context.Context, state *Peco, _ termbox.Event) {
+	if pdebug.Enabled {
+		g := pdebug.Marker("doGoToPreviousSelection")
+		defer g.End()
+	}
+
+	selection := state.Selection()
+
+	if selection.Len() == 0 {
+		state.Hub().SendStatusMsg(ctx, "No Selection")
+		return
+	}
+
+	b := state.CurrentLineBuffer()
+	l, err := b.LineAt(state.Location().LineNumber())
+	if err != nil {
+		return
+	}
+	currentLine := l.ID()
+	previousLine := uint64(0)
+	lastLine := uint64(math.MaxUint64)
+	found := false
+
+	selection.Ascend(func(it btree.Item) bool {
+		selectedLine := it.(line.Line)
+		if selectedLine.ID() < currentLine {
+			if selectedLine.ID() > previousLine {
+				previousLine = selectedLine.ID()
+				found = true
+			}
+		}
+
+		if selectedLine.ID() >= lastLine {
+			lastLine = selectedLine.ID()
+		}
+
+		return true
+	})
+
+	if found {
+		state.Hub().SendStatusMsg(ctx, "Previous Selection")
+		state.Hub().SendPaging(ctx, ToScrollFirstItem)
+		state.Hub().SendPaging(ctx, JumpToLineRequest(previousLine))
+	} else {
+		state.Hub().SendStatusMsg(ctx, "Previous Selection (first)")
+		state.Hub().SendPaging(ctx, ToScrollFirstItem)
+		state.Hub().SendPaging(ctx, JumpToLineRequest(lastLine))
+	}
+}
+
 func doSingleKeyJump(ctx context.Context, state *Peco, e termbox.Event) {
 	if pdebug.Enabled {
 		g := pdebug.Marker("doSingleKeyJump %c", e.Ch)
@@ -772,9 +908,9 @@ func doSingleKeyJump(ctx context.Context, state *Peco, e termbox.Event) {
 	}
 
 	toplevel, _ := ctx.Value(isTopLevelActionCall).(bool)
-	state.Hub().Batch(func() {
+	state.Hub().Batch(ctx, func(ctx context.Context) {
 		ctx = context.WithValue(ctx, isTopLevelActionCall, false)
-		state.Hub().SendPaging(JumpToLineRequest(index))
+		state.Hub().SendPaging(ctx, JumpToLineRequest(index))
 		doFinish(ctx, state, e)
 	}, toplevel)
 }
@@ -782,7 +918,7 @@ func doSingleKeyJump(ctx context.Context, state *Peco, e termbox.Event) {
 func makeCombinedAction(actions ...Action) ActionFunc {
 	return ActionFunc(func(ctx context.Context, state *Peco, e termbox.Event) {
 		toplevel, _ := ctx.Value(isTopLevelActionCall).(bool)
-		state.Hub().Batch(func() {
+		state.Hub().Batch(ctx, func(ctx context.Context) {
 			ctx = context.WithValue(ctx, isTopLevelActionCall, false)
 			for _, a := range actions {
 				a.Execute(ctx, state, e)
