@@ -6,14 +6,18 @@ func (c *Caret) Pos() int {
 	return c.pos
 }
 
+func (c *Caret) setPos_nolock(p int) {
+	c.pos = p
+}
+
 func (c *Caret) SetPos(p int) {
 	c.mutex.Lock()
 	defer c.mutex.Unlock()
-	c.pos = p
+	c.setPos_nolock(p)
 }
 
 func (c *Caret) Move(diff int) {
 	c.mutex.Lock()
 	defer c.mutex.Unlock()
-	c.pos = c.pos + diff
+	c.setPos_nolock(c.pos + diff)
 }
