@@ -3,8 +3,10 @@ package peco
 import (
 	"time"
 
-	"github.com/nsf/termbox-go"
 	"context"
+
+	"github.com/lestrrat-go/pdebug"
+	"github.com/nsf/termbox-go"
 )
 
 func NewInput(state *Peco, am ActionMap, src chan termbox.Event) *Input {
@@ -31,11 +33,16 @@ func (i *Input) Loop(ctx context.Context, cancel func()) error {
 }
 
 func (i *Input) handleInputEvent(ctx context.Context, ev termbox.Event) error {
+	if pdebug.Enabled {
+		g := pdebug.Marker("event received from user: %#v", ev)
+		defer g.End()
+	}
+
 	switch ev.Type {
 	case termbox.EventError:
 		return nil
 	case termbox.EventResize:
-		i.state.Hub().SendDraw(nil)
+		i.state.Hub().SendDraw(ctx, nil)
 		return nil
 	case termbox.EventKey:
 		// ModAlt is a sequence of letters with a leading \x1b (=Esc).
