@@ -22,13 +22,15 @@ const (
 )
 
 const (
-	ToLineAbove      PagingRequestType = iota // ToLineAbove moves the selection to the line above
-	ToScrollPageDown                          // ToScrollPageDown moves the selection to the next page
-	ToLineBelow                               // ToLineBelow moves the selection to the line below
-	ToScrollPageUp                            // ToScrollPageUp moves the selection to the previous page
-	ToScrollLeft                              // ToScrollLeft scrolls screen to the left
-	ToScrollRight                             // ToScrollRight scrolls screen to the right
-	ToLineInPage                              // ToLineInPage jumps to a particular line on the page
+	ToLineAbove       PagingRequestType = iota // ToLineAbove moves the selection to the line above
+	ToScrollPageDown                           // ToScrollPageDown moves the selection to the next page
+	ToLineBelow                                // ToLineBelow moves the selection to the line below
+	ToScrollPageUp                             // ToScrollPageUp moves the selection to the previous page
+	ToScrollLeft                               // ToScrollLeft scrolls screen to the left
+	ToScrollRight                              // ToScrollRight scrolls screen to the right
+	ToLineInPage                               // ToLineInPage jumps to a particular line on the page
+	ToScrollFirstItem                          // ToScrollFirstItem
+	ToScrollLastItem                           // ToScrollLastItem
 )
 
 const (
@@ -82,6 +84,7 @@ type Peco struct {
 	maxScanBufferSize       int
 	mutex                   sync.Mutex
 	onCancel                string
+	printQuery              bool
 	prompt                  string
 	query                   Query
 	queryExecDelay          time.Duration
@@ -100,6 +103,7 @@ type Peco struct {
 	singleKeyJumpShowPrefix bool
 	skipReadConfig          bool
 	styles                  StyleSet
+	use256Color             bool
 
 	// Source is where we buffer input. It gets reused when a new query is
 	// executed.
@@ -147,13 +151,14 @@ type Selection struct {
 // Screen hides termbox from the consuming code so that
 // it can be swapped out for testing
 type Screen interface {
-	Init() error
+	Init(*Config) error
 	Close() error
 	Flush() error
-	PollEvent(context.Context) chan termbox.Event
+	PollEvent(context.Context, *Config) chan termbox.Event
 	Print(PrintArgs) int
 	Resume()
 	SetCell(int, int, rune, termbox.Attribute, termbox.Attribute)
+	SetCursor(int, int)
 	Size() (int, int)
 	SendEvent(termbox.Event)
 	Suspend()
@@ -287,6 +292,7 @@ type Config struct {
 	Style               StyleSet          `json:"Style"`
 	Prompt              string            `json:"Prompt"`
 	Layout              string            `json:"Layout"`
+	Use256Color         bool              `json:"Use256Color"`
 	OnCancel            string            `json:"OnCancel"`
 	CustomMatcher       map[string][]string
 	CustomFilter        map[string]CustomFilterConfig
@@ -367,16 +373,18 @@ type FilterQuery Query
 type Source struct {
 	pipeline.ChanOutput
 
-	capacity  int
-	enableSep bool
-	idgen     line.IDGenerator
-	in        io.Reader
-	lines     []line.Line
-	name      string
-	mutex     sync.RWMutex
-	ready     chan struct{}
-	setupDone chan struct{}
-	setupOnce sync.Once
+	capacity   int
+	enableSep  bool
+	idgen      line.IDGenerator
+	in         io.Reader
+	inClosed   bool
+	isInfinite bool
+	lines      []line.Line
+	name       string
+	mutex      sync.RWMutex
+	ready      chan struct{}
+	setupDone  chan struct{}
+	setupOnce  sync.Once
 }
 
 type State interface {
@@ -407,6 +415,7 @@ type CLIOptions struct {
 	OptOnCancel        string `long:"on-cancel" description:"specify action on user cancel. 'success' or 'error'.\ndefault is 'success'. This may change in future versions"`
 	OptSelectionPrefix string `long:"selection-prefix" description:"use a prefix instead of changing line color to indicate currently selected lines.\ndefault is to use colors. This option is experimental"`
 	OptExec            string `long:"exec" description:"execute command instead of finishing/terminating peco.\nPlease note that this command will receive selected line(s) from stdin,\nand will be executed via '/bin/sh -c' or 'cmd /c'"`
+	OptPrintQuery      bool   `long:"print-query" descritpion:"print out the current query as first line of output"`
 }
 
 type CLI struct {
@@ -449,16 +458,16 @@ type Input struct {
 // message hub component. Unless we're in testing, github.com/peco/peco/hub.Hub
 // is used.
 type MessageHub interface {
-	Batch(func(), bool)
+	Batch(context.Context, func(context.Context), bool)
 	DrawCh() chan hub.Payload
 	PagingCh() chan hub.Payload
 	QueryCh() chan hub.Payload
-	SendDraw(interface{})
-	SendDrawPrompt()
-	SendPaging(interface{})
-	SendQuery(string)
-	SendStatusMsg(string)
-	SendStatusMsgAndClear(string, time.Duration)
+	SendDraw(context.Context, interface{})
+	SendDrawPrompt(context.Context)
+	SendPaging(context.Context, interface{})
+	SendQuery(context.Context, string)
+	SendStatusMsg(context.Context, string)
+	SendStatusMsgAndClear(context.Context, string, time.Duration)
 	StatusMsgCh() chan hub.Payload
 }
 
