@@ -9,6 +9,13 @@ type fder interface {
 	Fd() uintptr
 }
 
+func CaseInsensitiveIndexFunc(r rune) func(rune) bool {
+	lr := unicode.ToUpper(r)
+	return func(v rune) bool {
+		return lr == unicode.ToUpper(v)
+	}
+}
+
 func ContainsUpper(query string) bool {
 	for _, c := range query {
 		if unicode.IsUpper(c) {
@@ -38,6 +45,10 @@ type collectResults interface {
 	CollectResults() bool
 }
 
+type exitStatuser interface {
+	ExitStatus() int
+}
+
 func IsIgnorableError(err error) bool {
 	for e := err; e != nil; {
 		switch e.(type) {
@@ -65,3 +76,17 @@ func IsCollectResultsError(err error) bool {
 	}
 	return false
 }
+
+func GetExitStatus(err error) (int, bool) {
+	for e := err; e != nil; {
+		if ese, ok := e.(exitStatuser); ok {
+			return ese.ExitStatus(), true
+		}
+		if cerr, ok := e.(causer); ok {
+			e = cerr.Cause()
+			continue
+		}
+		break
+	}
+	return 1, false
+}
