@@ -7,7 +7,7 @@ import (
 	"time"
 	"unicode/utf8"
 
-	"github.com/lestrrat/go-pdebug"
+	"github.com/lestrrat-go/pdebug"
 	"github.com/mattn/go-runewidth"
 	"github.com/nsf/termbox-go"
 	"github.com/peco/peco/line"
@@ -111,6 +111,10 @@ func (u UserPrompt) Draw(state *Peco) {
 
 	fg := u.styles.Query.fg
 	bg := u.styles.Query.bg
+
+	// Used to notify termbox where our cursor is
+	var posX int
+
 	switch ql {
 	case 0:
 		u.screen.Print(PrintArgs{
@@ -120,6 +124,7 @@ func (u UserPrompt) Draw(state *Peco) {
 			Bg:   bg,
 			Fill: true,
 		})
+		posX = u.promptLen + 1
 		u.screen.Print(PrintArgs{
 			X:    u.promptLen + 1,
 			Y:    location,
@@ -145,8 +150,9 @@ func (u UserPrompt) Draw(state *Peco) {
 			Msg:  qs,
 			Fill: false,
 		})
+		posX = u.promptLen + 1 + int(runewidth.StringWidth(qs))
 		u.screen.Print(PrintArgs{
-			X:    u.promptLen + 1 + int(runewidth.StringWidth(qs)),
+			X:    posX,
 			Y:    location,
 			Fg:   fg | termbox.AttrReverse,
 			Bg:   bg | termbox.AttrReverse,
@@ -154,6 +160,7 @@ func (u UserPrompt) Draw(state *Peco) {
 			Fill: false,
 		})
 	default:
+		posX = c.Pos() + u.promptLen + 1
 		// the caret is in the middle of the string
 		prev := int(0)
 		var i int
@@ -179,6 +186,8 @@ func (u UserPrompt) Draw(state *Peco) {
 		})
 	}
 
+	u.screen.SetCursor(posX, location)
+
 	width, _ := u.screen.Size()
 
 	loc := state.Location()
@@ -758,6 +767,10 @@ func verticalScroll(state *Peco, l *BasicLayout, p PagingRequest) bool {
 			lineno -= lpp
 		case ToLineInPage:
 			lineno = loc.PerPage()*(loc.Page()-1) + p.(JumpToLineRequest).Line()
+		case ToScrollFirstItem:
+			lineno = 0
+		case ToScrollLastItem:
+			lineno = lcur - 1
 		}
 	} else {
 		switch p.Type() {
