@@ -1,11 +1,16 @@
+//go:generate stringer -type PagingRequestType -output stringer_paging_request_type.go .
+//go:generate stringer -type VerticalAnchor    -output stringer_vertical_anchor.go     .
+
 package peco
 
 import (
 	"bufio"
 	"bytes"
+	"fmt"
 	"io"
 	"os"
 	"reflect"
+	"runtime"
 	"sync"
 	"time"
 	"unicode/utf8"
@@ -13,7 +18,7 @@ import (
 	"context"
 
 	"github.com/google/btree"
-	"github.com/lestrrat/go-pdebug"
+	"github.com/lestrrat-go/pdebug"
 	"github.com/peco/peco/filter"
 	"github.com/peco/peco/hub"
 	"github.com/peco/peco/internal/util"
@@ -23,7 +28,7 @@ import (
 	"github.com/pkg/errors"
 )
 
-const version = "v0.5.1"
+const version = "v0.5.10"
 
 type errIgnorable struct {
 	err error
@@ -132,6 +137,10 @@ func (p *Peco) Styles() *StyleSet {
 	return &p.styles
 }
 
+func (p *Peco) Use256Color() bool {
+	return p.use256Color
+}
+
 func (p *Peco) Prompt() string {
 	return p.prompt
 }
@@ -203,7 +212,7 @@ func (p *Peco) SetSingleKeyJumpMode(b bool) {
 
 func (p *Peco) ToggleSingleKeyJumpMode() {
 	p.singleKeyJumpMode = !p.singleKeyJumpMode
-	go p.Hub().SendDraw(&DrawOptions{DisableCache: true})
+	go p.Hub().SendDraw(context.Background(), &DrawOptions{DisableCache: true})
 }
 
 func (p *Peco) SingleKeyJumpIndex(ch rune) (uint, bool) {
@@ -288,6 +297,20 @@ func (p *Peco) Setup() (err error) {
 	return nil
 }
 
+func (p *Peco) selectOneAndExitIfPossible() {
+	// TODO: mutex
+	// If we have only one line, we just want to bail out
+	// printing that one line as the result
+	if b := p.CurrentLineBuffer(); b.Size() == 1 {
+		if l, err := b.LineAt(0); err == nil {
+			p.resultCh = make(chan line.Line)
+			p.Exit(errCollectResults{})
+			p.resultCh <- l
+			close(p.resultCh)
+		}
+	}
+}
+
 func (p *Peco) Run(ctx context.Context) (err error) {
 	if pdebug.Enabled {
 		g := pdebug.Marker("Peco.Run").BindError(&err)
@@ -341,8 +364,8 @@ func (p *Peco) Run(ctx context.Context) (err error) {
 		// screen.Init must be called within Run() because we
 		// want to make sure to call screen.Close() after getting
 		// out of Run()
-		p.screen.Init()
-		go NewInput(p, p.Keymap(), p.screen.PollEvent(ctx)).Loop(ctx, cancel)
+		p.screen.Init(&p.config)
+		go NewInput(p, p.Keymap(), p.screen.PollEvent(ctx, &p.config)).Loop(ctx, cancel)
 		go NewView(p).Loop(ctx, cancel)
 		go NewFilter(p).Loop(ctx, cancel)
 	}()
@@ -366,16 +389,7 @@ func (p *Peco) Run(ctx context.Context) (err error) {
 			// a line, where as SetupDone waits until we're completely
 			// done reading the input
 			<-p.source.SetupDone()
-			// If we have only one line, we just want to bail out
-			// printing that one line as the result
-			if b := p.CurrentLineBuffer(); b.Size() == 1 {
-				if l, err := b.LineAt(0); err == nil {
-					p.resultCh = make(chan line.Line)
-					p.Exit(errCollectResults{})
-					p.resultCh <- l
-					close(p.resultCh)
-				}
-			}
+			p.selectOneAndExitIfPossible()
 		}()
 	}
 
@@ -391,7 +405,14 @@ func (p *Peco) Run(ctx context.Context) (err error) {
 	if p.Query().Len() > 0 {
 		go func() {
 			<-p.source.Ready()
-			p.ExecQuery()
+
+			// iff p.selectOneAndExit is true, we should check after exec query is run
+			// if we only have one item
+			if p.selectOneAndExit {
+				p.ExecQuery(p.selectOneAndExitIfPossible)
+			} else {
+				p.ExecQuery(nil)
+			}
 		}()
 	}
 
@@ -416,7 +437,7 @@ func (p *Peco) parseCommandLine(opts *CLIOptions, args *[]string, argv []string)
 	}
 
 	if opts.OptVersion {
-		p.Stdout.Write([]byte("peco version " + version + "\n"))
+		fmt.Fprintf(p.Stdout, "peco version %s (built with %s)\n", version, runtime.Version())
 		return makeIgnorable(errors.New("user asked to show version"))
 	}
 
@@ -439,6 +460,7 @@ func (p *Peco) SetupSource(ctx context.Context) (s *Source, err error) {
 
 	var in io.Reader
 	var filename string
+	var isInfinite bool
 	switch {
 	case len(p.args) > 1:
 		f, err := os.Open(p.args[1])
@@ -456,11 +478,16 @@ func (p *Peco) SetupSource(ctx context.Context) (s *Source, err error) {
 		}
 		in = p.Stdin
 		filename = `-`
+		// XXX we detect that this is potentially an "infinite" source if
+		// the input is coming from Stdin. This is important b/c we need to
+		// know NOT to use batch mode processing when the incoming source
+		// is never-ending
+		isInfinite = true
 	default:
 		return nil, errors.New("you must supply something to work with via filename or stdin")
 	}
 
-	src := NewSource(filename, in, p.idgen, p.bufferSize, p.enableSep)
+	src := NewSource(filename, in, isInfinite, p.idgen, p.bufferSize, p.enableSep)
 
 	// Block until we receive something from `in`
 	if pdebug.Enabled {
@@ -519,6 +546,8 @@ func (p *Peco) ApplyConfig(opts CLIOptions) error {
 		p.prompt = v
 	}
 
+	p.use256Color = p.config.Use256Color
+
 	p.onCancel = successKey
 	if opts.OptOnCancel == errorKey || p.config.OnCancel == errorKey {
 		p.onCancel = errorKey
@@ -530,6 +559,7 @@ func (p *Peco) ApplyConfig(opts CLIOptions) error {
 		p.selectionPrefix = p.config.SelectionPrefix
 	}
 	p.selectOneAndExit = opts.OptSelect1
+	p.printQuery = opts.OptPrintQuery
 	p.initialQuery = opts.OptQuery
 	p.initialFilter = opts.OptInitialFilter
 	if len(p.initialFilter) <= 0 {
@@ -632,19 +662,51 @@ func (p *Peco) SetCurrentLineBuffer(b Buffer) {
 		defer g.End()
 	}
 	p.currentLineBuffer = b
-	go p.Hub().SendDraw(nil)
+	go p.Hub().SendDraw(context.Background(), nil)
 }
 
 func (p *Peco) ResetCurrentLineBuffer() {
 	p.SetCurrentLineBuffer(p.source)
 }
 
-func (p *Peco) ExecQuery() bool {
+func (p *Peco) sendQuery(ctx context.Context, q string, nextFunc func()) {
+	if pdebug.Enabled {
+		g := pdebug.Marker("sending query to filter goroutine (q=%v, isInfinite=%t)", q, p.source.IsInfinite())
+		defer g.End()
+	}
+
+	if p.source.IsInfinite() {
+		// If the source is a stream, we can't do batch mode, and hence
+		// we can't guarantee proper timing. But... okay, we simulate
+		// something like it
+		p.Hub().SendQuery(ctx, q)
+		if nextFunc != nil {
+			time.AfterFunc(time.Second, nextFunc)
+		}
+	} else {
+		// No delay, execute immediately
+		p.Hub().Batch(context.Background(), func(ctx context.Context) {
+			p.Hub().SendQuery(ctx, q)
+			if nextFunc != nil {
+				nextFunc()
+			}
+		}, false)
+	}
+}
+
+// ExecQuery executes the query, taking in consideration things like the
+// exec-delay, and user's multiple successive inputs in a very short span
+//
+// if nextFunc is non-nil, then nextFunc is executed after the query is
+// executed
+func (p *Peco) ExecQuery(nextFunc func()) bool {
 	if pdebug.Enabled {
 		g := pdebug.Marker("Peco.ExecQuery")
 		defer g.End()
 	}
 
+	hub := p.Hub()
+
 	select {
 	case <-p.Ready():
 	default:
@@ -662,7 +724,13 @@ func (p *Peco) ExecQuery() bool {
 			pdebug.Printf("empty query, reset buffer")
 		}
 		p.ResetCurrentLineBuffer()
-		p.Hub().SendDraw(&DrawOptions{DisableCache: true})
+
+		hub.Batch(context.Background(), func(ctx context.Context) {
+			hub.SendDraw(ctx, &DrawOptions{DisableCache: true})
+			if nextFunc != nil {
+				nextFunc()
+			}
+		}, false)
 		return true
 	}
 
@@ -671,8 +739,8 @@ func (p *Peco) ExecQuery() bool {
 		if pdebug.Enabled {
 			pdebug.Printf("sending query (immediate)")
 		}
-		// No delay, execute immediately
-		p.Hub().SendQuery(q.String())
+
+		p.sendQuery(context.Background(), q.String(), nextFunc)
 		return true
 	}
 
@@ -689,13 +757,17 @@ func (p *Peco) ExecQuery() bool {
 	// Wait $delay millisecs before sending the query
 	// if a new input comes in, batch them up
 	if pdebug.Enabled {
-		pdebug.Printf("sending query with delay)")
+		pdebug.Printf("sending query (with delay)")
 	}
 	p.queryExecTimer = time.AfterFunc(delay, func() {
 		if pdebug.Enabled {
 			pdebug.Printf("delayed query sent")
 		}
-		p.Hub().SendQuery(q.String())
+		p.sendQuery(context.Background(), q.String(), nextFunc)
+
+		if pdebug.Enabled {
+			pdebug.Printf("delayed query executed")
+		}
 
 		p.queryExecMutex.Lock()
 		defer p.queryExecMutex.Unlock()
@@ -726,6 +798,14 @@ func (p *Peco) PrintResults() {
 	}()
 
 	var buf bytes.Buffer
+
+	if pdebug.Enabled {
+		pdebug.Printf("--print-query was %t", p.printQuery)
+	}
+	if p.printQuery {
+		buf.WriteString(p.Query().String())
+		buf.WriteByte('\n')
+	}
 	for line := range p.ResultCh() {
 		buf.WriteString(line.Output())
 		buf.WriteByte('\n')
