@@ -12,6 +12,7 @@ import (
 
 	"context"
 
+	"github.com/lestrrat-go/pdebug"
 	"github.com/nsf/termbox-go"
 	"github.com/peco/peco/hub"
 	"github.com/peco/peco/internal/util"
@@ -21,17 +22,17 @@ import (
 
 type nullHub struct{}
 
-func (h nullHub) Batch(_ func(), _ bool)                          {}
-func (h nullHub) DrawCh() chan hub.Payload                        { return nil }
-func (h nullHub) PagingCh() chan hub.Payload                      { return nil }
-func (h nullHub) QueryCh() chan hub.Payload                       { return nil }
-func (h nullHub) SendDraw(_ interface{})                          {}
-func (h nullHub) SendDrawPrompt()                                 {}
-func (h nullHub) SendPaging(_ interface{})                        {}
-func (h nullHub) SendQuery(_ string)                              {}
-func (h nullHub) SendStatusMsg(_ string)                          {}
-func (h nullHub) SendStatusMsgAndClear(_ string, _ time.Duration) {}
-func (h nullHub) StatusMsgCh() chan hub.Payload                   { return nil }
+func (h nullHub) Batch(_ context.Context, _ func(context.Context), _ bool)           {}
+func (h nullHub) DrawCh() chan hub.Payload                                           { return nil }
+func (h nullHub) PagingCh() chan hub.Payload                                         { return nil }
+func (h nullHub) QueryCh() chan hub.Payload                                          { return nil }
+func (h nullHub) SendDraw(_ context.Context, _ interface{})                          {}
+func (h nullHub) SendDrawPrompt(context.Context)                                     {}
+func (h nullHub) SendPaging(_ context.Context, _ interface{})                        {}
+func (h nullHub) SendQuery(_ context.Context, _ string)                              {}
+func (h nullHub) SendStatusMsg(_ context.Context, _ string)                          {}
+func (h nullHub) SendStatusMsgAndClear(_ context.Context, _ string, _ time.Duration) {}
+func (h nullHub) StatusMsgCh() chan hub.Payload                                      { return nil }
 
 type interceptorArgs []interface{}
 type interceptor struct {
@@ -101,7 +102,10 @@ func NewDummyScreen() *dummyScreen {
 	}
 }
 
-func (d dummyScreen) Init() error {
+func (d dummyScreen) SetCursor(_, _ int) {
+}
+
+func (d dummyScreen) Init(cfg *Config) error {
 	return nil
 }
 
@@ -114,7 +118,14 @@ func (d dummyScreen) Print(args PrintArgs) int {
 }
 
 func (d dummyScreen) SendEvent(e termbox.Event) {
-	d.pollCh <- e
+	// XXX FIXME SendEvent should receive a context
+	t := time.NewTimer(time.Second)
+	defer t.Stop()
+	select {
+	case <-t.C:
+		panic("timed out sending an event")
+	case d.pollCh <- e:
+	}
 }
 
 func (d dummyScreen) SetCell(x, y int, ch rune, fg, bg termbox.Attribute) {
@@ -124,13 +135,13 @@ func (d dummyScreen) Flush() error {
 	d.record("Flush", interceptorArgs{})
 	return nil
 }
-func (d dummyScreen) PollEvent(ctx context.Context) chan termbox.Event {
+func (d dummyScreen) PollEvent(ctx context.Context, cfg *Config) chan termbox.Event {
 	return d.pollCh
 }
 func (d dummyScreen) Size() (int, int) {
 	return d.width, d.height
 }
-func (d dummyScreen) Resume() {}
+func (d dummyScreen) Resume()  {}
 func (d dummyScreen) Suspend() {}
 
 func TestIDGen(t *testing.T) {
@@ -229,6 +240,7 @@ func TestApplyConfig(t *testing.T) {
 	opts.OptSelect1 = true
 	opts.OptOnCancel = "error"
 	opts.OptSelectionPrefix = ">"
+	opts.OptPrintQuery = true
 
 	p := newPeco()
 	if !assert.NoError(t, p.ApplyConfig(opts), "p.ApplyConfig should succeed") {
@@ -274,6 +286,9 @@ func TestApplyConfig(t *testing.T) {
 	if !assert.Equal(t, opts.OptSelectionPrefix, p.selectionPrefix, "p.selectionPrefix should be equal to opts.OptSelectionPrefix") {
 		return
 	}
+	if !assert.Equal(t, opts.OptPrintQuery, p.printQuery, "p.printQuery should be equal to opts.OptPrintQuery") {
+		return
+	}
 }
 
 // While this issue is labeled for Issue363, it tests against 376 as well.
@@ -308,9 +323,10 @@ func TestGHIssue363(t *testing.T) {
 		if !assert.True(t, util.IsCollectResultsError(err), "isCollectResultsError") {
 			return
 		}
+		p.PrintResults()
 	}
 
-	if !assert.NotEqual(t, "foo\n", out.String(), "output should match") {
+	if !assert.Equal(t, "foo\n", out.String(), "output should match") {
 		return
 	}
 }
@@ -344,7 +360,11 @@ func TestGHIssue367(t *testing.T) {
 
 		l := len(src[0])
 		copy(p, src[0])
+		p = p[:l]
 		src = src[1:]
+		if pdebug.Enabled {
+			pdebug.Printf("reader func returning %#v", string(p))
+		}
 		return l, nil
 	})
 	buf := bytes.Buffer{}
@@ -356,9 +376,13 @@ func TestGHIssue367(t *testing.T) {
 		p.Run(ctx)
 	}()
 
-	p.Query().Set("bar")
-
 	select {
+	case <-time.After(100 * time.Millisecond):
+		p.screen.SendEvent(termbox.Event{Ch: 'b'})
+	case <-time.After(200 * time.Millisecond):
+		p.screen.SendEvent(termbox.Event{Ch: 'a'})
+	case <-time.After(300 * time.Millisecond):
+		p.screen.SendEvent(termbox.Event{Ch: 'r'})
 	case <-time.After(900 * time.Millisecond):
 		p.screen.SendEvent(termbox.Event{Key: termbox.KeyEnter})
 	}
@@ -384,3 +408,84 @@ func TestGHIssue367(t *testing.T) {
 		return
 	}
 }
+
+func TestPrintQuery(t *testing.T) {
+	t.Run("Match and print query", func(t *testing.T) {
+		ctx, cancel := context.WithTimeout(context.Background(), time.Second)
+		defer cancel()
+
+		p := newPeco()
+		p.Argv = []string{"--print-query", "--query", "oo", "--select-1"}
+		p.Stdin = bytes.NewBufferString("foo\n")
+		var out bytes.Buffer
+		p.Stdout = &out
+
+		resultCh := make(chan error)
+		go func() {
+			defer close(resultCh)
+			select {
+			case <-ctx.Done():
+				return
+			case resultCh <- p.Run(ctx):
+				return
+			}
+		}()
+
+		select {
+		case <-ctx.Done():
+			t.Errorf("timeout reached")
+			return
+		case err := <-resultCh:
+			if !assert.True(t, util.IsCollectResultsError(err), "isCollectResultsError") {
+				return
+			}
+			p.PrintResults()
+		}
+
+		if !assert.Equal(t, "oo\nfoo\n", out.String(), "output should match") {
+			return
+		}
+	})
+	t.Run("No match and print query", func(t *testing.T) {
+		ctx, cancel := context.WithTimeout(context.Background(), 5*time.Second)
+		defer cancel()
+
+		p := newPeco()
+		p.Argv = []string{"--print-query", "--query", "oo"}
+		p.Stdin = bytes.NewBufferString("bar\n")
+		var out bytes.Buffer
+		p.Stdout = &out
+
+		resultCh := make(chan error)
+		go func() {
+			defer close(resultCh)
+			select {
+			case <-ctx.Done():
+				return
+			case resultCh <- p.Run(ctx):
+				return
+			}
+		}()
+
+		<-p.Ready()
+
+		time.AfterFunc(100*time.Millisecond, func() {
+			p.screen.SendEvent(termbox.Event{Key: termbox.KeyEnter})
+		})
+
+		select {
+		case <-ctx.Done():
+			t.Errorf("timeout reached")
+			return
+		case err := <-resultCh:
+			if !assert.True(t, util.IsCollectResultsError(err), "isCollectResultsError") {
+				return
+			}
+			p.PrintResults()
+		}
+
+		if !assert.Equal(t, "oo\n", out.String(), "output should match") {
+			return
+		}
+	})
+}
