@@ -4,18 +4,18 @@ import (
 	"context"
 	"unicode/utf8"
 
-	pdebug "github.com/lestrrat/go-pdebug"
+	pdebug "github.com/lestrrat-go/pdebug"
 	"github.com/mattn/go-runewidth"
 	"github.com/nsf/termbox-go"
 	"github.com/pkg/errors"
 )
 
-func (t *Termbox) Init() error {
+func (t *Termbox) Init(cfg *Config) error {
 	if err := termbox.Init(); err != nil {
 		return errors.Wrap(err, "failed to initialized termbox")
 	}
 
-	return t.PostInit()
+	return t.PostInit(cfg)
 }
 
 func NewTermbox() *Termbox {
@@ -34,6 +34,10 @@ func (t *Termbox) Close() error {
 	return nil
 }
 
+func (t *Termbox) SetCursor(x, y int) {
+	termbox.SetCursor(x, y)
+}
+
 // SendEvent is used to allow programmers generate random
 // events, but it's only useful for testing purposes.
 // When interactiving with termbox-go, this method is a noop
@@ -51,7 +55,7 @@ func (t *Termbox) Flush() error {
 // PollEvent returns a channel that you can listen to for
 // termbox's events. The actual polling is done in a
 // separate gouroutine
-func (t *Termbox) PollEvent(ctx context.Context) chan termbox.Event {
+func (t *Termbox) PollEvent(ctx context.Context, cfg *Config) chan termbox.Event {
 	// XXX termbox.PollEvent() can get stuck on unexpected signal
 	// handling cases. We still would like to wait until the user
 	// (termbox) has some event for us to process, but we don't
@@ -93,7 +97,7 @@ func (t *Termbox) PollEvent(ctx context.Context) chan termbox.Event {
 			case <-ctx.Done():
 				return
 			case replyCh := <-t.resumeCh:
-				t.Init()
+				t.Init(cfg)
 				close(replyCh)
 			}
 		}
